#include "paths.hpp"
#include "config.hpp"
#include "whereami.h"

#include <paths.h>

namespace myx {

namespace filesystem {

Paths::Paths() :
	m_prefixDirectory( "/opt/" ORGANIZATION_NAME_LOWER "/" PROJECT_NAME_LOWER ),
	m_binaryDirectory( m_prefixDirectory.absolutePath() + "/bin" ),
	m_configDirectory( m_prefixDirectory.absolutePath() + "/etc" ),
	m_cacheDirectory( m_prefixDirectory.absolutePath() + "/var" ),
	m_tempDirectory( QString::fromLocal8Bit( qgetenv( qPrintable( "TMPDIR" ) ) ) ),
	m_dataDirectory( m_prefixDirectory.absolutePath() + "/share" ),
	m_homeDirectory( QString::fromLocal8Bit( qgetenv( qPrintable( "HOME" ) ) ) ),
	m_executableFile( m_binaryDirectory.absolutePath() + "/" + PROJECT_NAME_LOWER ),
	m_configFile( m_configDirectory.absolutePath() + "/" + PROJECT_NAME_LOWER + ".conf" )
{
	if ( m_tempDirectory.isEmpty() || ( m_tempDirectory.path() == "." ) )
	{
		m_tempDirectory = _PATH_TMP;
	}
}


QDir Paths::prefixDirectory() const
{
	return( m_prefixDirectory );
}


void Paths::setPrefixDirectory( const QString& prefixDirectory )
{
	m_prefixDirectory = prefixDirectory;
}


QDir Paths::binaryDirectory() const
{
	return( m_binaryDirectory );
}


void Paths::setBinaryDirectory( const QString& binaryDirectory )
{
	m_binaryDirectory = binaryDirectory;
}


QDir Paths::configDirectory() const
{
	return( m_configDirectory );
}


void Paths::setConfigDirectory( const QString& configDirectory )
{
	m_configDirectory = configDirectory;
}


QDir Paths::cacheDirectory() const
{
	return( m_cacheDirectory );
}


void Paths::setCacheDirectory( const QString& cacheDirectory )
{
	m_cacheDirectory = cacheDirectory;
}


QDir Paths::tempDirectory() const
{
	return( m_tempDirectory );
}


void Paths::setTempDirectory( const QString& tempDirectory )
{
	m_tempDirectory = tempDirectory;
}


QDir Paths::dataDirectory() const
{
	return( m_dataDirectory );
}


void Paths::setDataDirectory( const QString& dataDirectory )
{
	m_dataDirectory = dataDirectory;
}


QFileInfo Paths::executableFile() const
{
	return( m_executableFile );
}


void Paths::setExecutableFile( const QString& executableFile )
{
	m_executableFile = executableFile;
}


QFileInfo Paths::configFile() const
{
	return( m_configFile );
}


void Paths::setConfigFile( const QString& configFile )
{
	m_configFile = configFile;
}


bool Paths::updatePaths()
{
	char* path = nullptr;
	int fullPathLength;
	int dirNameLength;

	fullPathLength = wai_getExecutablePath( nullptr, 0, &dirNameLength );
	if ( fullPathLength > 0 )
	{
		path = new( std::nothrow ) char[static_cast< size_t >( fullPathLength ) + 1 ];
		if ( path == nullptr )
		{
			return( false );
		}
		wai_getExecutablePath( path, fullPathLength, &dirNameLength );
		path[fullPathLength] = '\0';
	}
	else
	{
		return( false );
	}

	m_executableFile = QFile( path );
	path[dirNameLength] = '\0';
	m_binaryDirectory = path;
	delete [] path;

	if ( m_binaryDirectory.absolutePath().endsWith( "/bin" ) )
	{
		m_prefixDirectory = m_binaryDirectory.absolutePath().remove( QRegExp( "/bin$" ) );
		m_configDirectory = m_prefixDirectory.absolutePath() + "/etc";
		m_cacheDirectory = m_prefixDirectory.absolutePath() + "/var";
		m_dataDirectory = m_prefixDirectory.absolutePath() + "/share";
		m_configFile = m_configDirectory.absolutePath() + "/" + PROJECT_NAME_LOWER + ".conf";
	}

	if ( m_prefixDirectory.absolutePath().startsWith( "/opt" ) ||
	     m_prefixDirectory.absolutePath().startsWith( "/usr" ) )
	{
		QString dataDirectory = QString::fromLocal8Bit( qgetenv( qPrintable( "XDG_DATA_HOME" ) ) );
		if ( dataDirectory.isEmpty() )
		{
			dataDirectory = m_homeDirectory.absolutePath() + ".local/share/";
		}
		m_dataDirectory = dataDirectory + ORGANIZATION_NAME_LOWER + "/" + PROJECT_NAME_LOWER;

		QString configDirectory = QString::fromLocal8Bit( qgetenv( qPrintable( "XDG_CONFIG_HOME" ) ) );
		if ( configDirectory.isEmpty() )
		{
			configDirectory = m_homeDirectory.absolutePath() + ".config/";
		}
		m_configDirectory = configDirectory + ORGANIZATION_NAME_LOWER + "/" + PROJECT_NAME_LOWER;

		QString cacheDirectory = QString::fromLocal8Bit( qgetenv( qPrintable( "XDG_CACHE_HOME" ) ) );
		if ( cacheDirectory.isEmpty() )
		{
			cacheDirectory = m_homeDirectory.absolutePath() + ".cache/";
		}
		m_cacheDirectory = cacheDirectory + ORGANIZATION_NAME_LOWER + "/" + PROJECT_NAME_LOWER;
	}

	return( true );
} // Paths::updatePaths


bool Paths::makePaths()
{
	bool status = true;

	if ( !m_dataDirectory.mkpath( m_dataDirectory.absolutePath() ) ) { status = false; }
	if ( !m_configDirectory.mkpath( m_configDirectory.absolutePath() ) ) { status = false; }
	if ( !m_cacheDirectory.mkpath( m_cacheDirectory.absolutePath() ) ) { status = false; }
	return( status );
}


QString Paths::findConfigFile( const QString& defaultConfigFile )
{
	if ( QFileInfo( defaultConfigFile ).isReadable() )
	{
		return( defaultConfigFile );
	}

	auto fileName = QString::fromLocal8Bit( qgetenv( qPrintable( PROJECT_NAME_UPPER "_CONFIG" ) ) );
	if ( QFileInfo( fileName ).isReadable() )
	{
		return( fileName );
	}

	if ( QFileInfo( m_configFile ).isReadable() )
	{
		return( m_configFile.absoluteFilePath() );
	}

	return( QString() );
}

} // namespace filesystem

} // namespace myx
